@@ -1,6 +1,7 @@
 /*
     This file is part of Rocs.
     Copyright 2011  Wagner Reck <wagner.reck@gmail.com>
+    Copyright 2013  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -17,10 +18,10 @@
 */
 
 #include "ListStructure.h"
-#include "KDebug"
 #include "ListNode.h"
 #include "Pointer.h"
-#include <QDebug>
+#include <KLocale>
+#include <KDebug>
 #include <boost/shared_ptr.hpp>
 
 
@@ -36,7 +37,6 @@ DataStructurePtr Rocs::ListStructure::create(DataStructurePtr other, Document *p
     return ds;
 }
 
-
 Rocs::ListStructure::ListStructure(Document* parent)
     : DataStructure(parent)
     , m_building(true)
@@ -51,7 +51,7 @@ void Rocs::ListStructure::importStructure(DataStructurePtr other)
     QHash < Data*, DataPtr > dataTodata;
     foreach (int type, other->document()->dataTypeList()) {
         foreach (DataPtr n, other->dataList(type)) {
-            DataPtr newdata = addData("", type);
+            DataPtr newdata = createData("", type);
             newdata->setColor(n->color());
             newdata->setProperty("value", n->property("value").toString());
             newdata->setX(n->x());
@@ -59,7 +59,7 @@ void Rocs::ListStructure::importStructure(DataStructurePtr other)
             newdata->setWidth(n->width());
             dataTodata.insert(n.get(), newdata);
             foreach (const QString &property, other->document()->dataType(type)->properties()) {
-                newdata->setProperty(property.toStdString().c_str(), n->property(property.toStdString().c_str()));
+                newdata->setProperty(property.toLatin1(), n->property(property.toLatin1()));
             }
         }
     }
@@ -68,11 +68,11 @@ void Rocs::ListStructure::importStructure(DataStructurePtr other)
             DataPtr from =  dataTodata.value(e->from().get());
             DataPtr to =  dataTodata.value(e->to().get());
 
-            PointerPtr newPointer = addPointer(from, to, type);
+            PointerPtr newPointer = createPointer(from, to, type);
             newPointer->setColor(e->color());
             newPointer->setProperty("value", e->property("value").toString());
             foreach (const QString &property, other->document()->pointerType(type)->properties()) {
-                newPointer->setProperty(property.toStdString().c_str(), e->property(property.toStdString().c_str()));
+                newPointer->setProperty(property.toLatin1(), e->property(property.toLatin1()));
             }
         }
     }
@@ -83,58 +83,44 @@ void Rocs::ListStructure::importStructure(DataStructurePtr other)
 
 void Rocs::ListStructure::init()
 {
+    connect(this, SIGNAL(changed()), this, SLOT(arrangeNodes()));
+
     m_animationGroup = new QParallelAnimationGroup(parent());
-    if (!dataList().isEmpty()) {
-        m_begin = boost::static_pointer_cast<ListNode>(dataList().first());
-        arrangeNodes();
-    } else {
-        m_begin = boost::shared_ptr<ListNode>();
+    if (!dataList(0).isEmpty()) {
+        m_begin = boost::static_pointer_cast<ListNode>(dataList(0).first());
     }
 }
 
-
 Rocs::ListStructure::~ListStructure()
 {
+    m_building = true;
+    m_animationGroup->stop();
 // FIXME this is a memory leak, but fix this later
 // since delete later guides to crash on deletion
 //     m_animationGroup->deleteLater();
 }
 
-PointerPtr Rocs::ListStructure::addPointer(DataPtr from, DataPtr to, int pointerType)
+PointerPtr Rocs::ListStructure::createPointer(DataPtr from, DataPtr to, int pointerType)
 {
     foreach(PointerPtr e, from->outPointerList()) {
         e->remove();
     }
 
-    PointerPtr e = DataStructure::addPointer(from, to, pointerType);
-    arrangeNodes();
+    PointerPtr e = DataStructure::createPointer(from, to, pointerType);
     return e;
 }
 
-DataPtr Rocs::ListStructure::addData(const QString& name, int dataType)
+DataPtr Rocs::ListStructure::createData(const QString& name, int dataType)
 {
     boost::shared_ptr<ListNode> n = boost::static_pointer_cast<ListNode>(
                                         ListNode::create(getDataStructure(), generateUniqueIdentifier(), dataType)
                                     );
     n->setProperty("name", name);
+    addData(n);
 
-    if (m_building) {
-        return addData(n, dataType);;
-    }
-
-    addData(n, dataType);
-    if (m_begin) {
-        boost::shared_ptr<ListNode> tmp = m_begin;
-        while (tmp->next()) {
-            tmp = tmp->next();
-        }
-        if (tmp) {
-            tmp->pointTo(n);
-        }
-    } else {
-        m_begin = n;
-    }
-    arrangeNodes();
+    // insert node as new head
+    n->pointTo(m_begin);
+    m_begin = n;
     return n;
 }
 
@@ -143,56 +129,74 @@ void Rocs::ListStructure::remove(DataPtr n)
     if (m_begin == n) {
         m_begin = boost::static_pointer_cast<ListNode>(n)->next();
     }
-    foreach(PointerPtr p, n->inPointerList()) {
-        p->remove();
-    }
     DataStructure::remove(n);
-    arrangeNodes();
 }
 
-
-QScriptValue Rocs::ListStructure::begin()
+QScriptValue Rocs::ListStructure::head()
 {
     return m_begin->scriptValue();
 }
 
-void Rocs::ListStructure::setBegin(Data* node)
+void Rocs::ListStructure::setHead(Data *headNode)
 {
-    m_begin = boost::static_pointer_cast<ListNode>(node->getData());
-    arrangeNodes();
+    Q_ASSERT(headNode);
+    if (!headNode) {
+        return;
+    }
+    //FIXME: we need resorting!
+    m_begin = boost::static_pointer_cast<ListNode>(headNode->getData());
 }
 
+QScriptValue Rocs::ListStructure::begin()
+{
+    emit scriptError(i18n("The method \"%1\" is deprecated, please use \"%2\" instead.",
+        QString("begin()"),
+        QString("head()")));
+    return head();
+}
+
+void Rocs::ListStructure::setBegin(Data* node)
+{
+    emit scriptError(i18n("The method \"%1\" is deprecated, please use \"%2\" instead.",
+        QString("setBegin(node)"),
+        QString("setHead(node)")));
+    setHead(node);
+}
 
 void Rocs::ListStructure::remove(PointerPtr ptr)
 {
     DataStructure::remove(ptr);
-    arrangeNodes();
 }
 
-QScriptValue Rocs::ListStructure::createNode(const QString& name)
+QScriptValue Rocs::ListStructure::createNode()
+{
+    return createNode(0);
+}
+
+QScriptValue Rocs::ListStructure::createNode(int type)
 {
     boost::shared_ptr<ListNode> n = boost::static_pointer_cast<ListNode>(
-                                        DataStructure::addData(ListNode::create(getDataStructure(), generateUniqueIdentifier(), 0))
+                                        DataStructure::addData(ListNode::create(getDataStructure(), generateUniqueIdentifier(), type))
                                     );
-    n->setProperty("name", name);
     n->setEngine(engine());
-    arrangeNodes();
     return n->scriptValue();
 }
 
 void Rocs::ListStructure::arrangeNodes()
 {
-    if (m_building)
+    if (m_building) {
         return;
+    }
 
-    QRectF size = document()->size();
+    QRectF size = document()->sceneRect();
     qreal x;
     qreal y = size.top() + 100;;
     if (m_animationGroup->state() != QAnimationGroup::Stopped) {
         m_animationGroup->stop();
     }
-    QScopedArrayPointer<bool>visited(new bool[dataList().count()]);
-    for (int i = 0; i < dataList().count(); ++i) {
+    //FIXME only default data type considered
+    QScopedArrayPointer<bool>visited(new bool[dataList(0).count()]);
+    for (int i = 0; i < dataList(0).count(); ++i) {
         visited[i] = false;
     }
 
@@ -201,10 +205,10 @@ void Rocs::ListStructure::arrangeNodes()
     if (n) {
         x = size.left() + 50;
         do {
-            if (visited[dataList().indexOf(n)]) {
+            if (visited[dataList(0).indexOf(n)]) {
                 break;
             }
-            visited[dataList().indexOf(n)] = true;
+            visited[dataList(0).indexOf(n)] = true;
             if (x > size.right() - 140 + n->width() * 40) {
                 x = size.left() + 50 + n->width() * 40;
                 y += 85;
@@ -225,8 +229,9 @@ void Rocs::ListStructure::arrangeNodes()
     }
     x = 50 + size.left();
     y += 100;
-    foreach(DataPtr n, dataList()) {
-        if (!visited[dataList().indexOf(n)]) {
+    //FIXME only default data type considered
+    foreach(DataPtr n, dataList(0)) {
+        if (!visited[dataList(0).indexOf(n)]) {
             anim = new QPropertyAnimation(n.get(), "x");;
             anim->setDuration(500);
             anim->setStartValue(n->x());
